import jssc.SerialPort;
import jssc.SerialPortEvent;
import jssc.SerialPortEventListener;
import jssc.SerialPortException;
import jssc.SerialPortList;

import org.json.JSONObject;

public class Main {
	
	static SerialPort serialPort;
	/**
	 * @param args
	 * @throws InterruptedException 
	 * @throws SerialPortException 
	 */
	public static void main(String[] args) throws InterruptedException, SerialPortException {

		/*
			{
			    "inventory": "bool",
			    "mode": "int",
			    "session": "int",
			    "profile": "int",
			    "population": "int",
			    "fast_id": "bool",
			    "rssi": "bool",
			    "phase": "bool",
			    "channel": "bool",
			    "antenna": "bool",
			    "reader_name": "bool",
			    "timestamp": "bool",
			    "input_epc": "string",
			    "output_epc": "string",
			    "buzzer": "int",
			    "led": "int",
			    "tx1" : "int,
			    "reset" : "bool"
			}
		*/
		
		try 
		{
			String[] portNames = SerialPortList.getPortNames();
			serialPort = new SerialPort(portNames[0]);

			
			serialPort.openPort();	

		    serialPort.setParams(SerialPort.BAUDRATE_115200,
		                         SerialPort.DATABITS_8,
		                         SerialPort.STOPBITS_1,
		                         SerialPort.PARITY_NONE);

		    serialPort.setFlowControlMode(SerialPort.FLOWCONTROL_RTSCTS_IN | 
		                               		    		SerialPort.FLOWCONTROL_RTSCTS_OUT);

		    int mask = SerialPort.MASK_RXCHAR;
		    serialPort.addEventListener(new PortReader(), mask);
		    


		    // How to send change configurations
		    // build a json string with \r\r at the end and send it to the USB PORT
		    // each configuration must be sent once, avoid sending two configuration at the same time
		    
		    JSONObject file = new JSONObject();
		    
		    // epc write
//		    file.put("input_epc", "121212345678987645760000");
//		    file.put("output_epc", "121212345678987645760000");
		    
		    String toSendOverUSB = file.toString().concat("\r\r");
		    
//		    serialPort.writeString(toSendOverUSB);
		    
		    // start inventory
		    file.clear();
		    file.put("reset", true);
		    toSendOverUSB = file.toString().concat("\r\r");
//		    serialPort.writeString(toSendOverUSB);
		    
		    
		    		    
		    // mechanism to detect usb disconnection 
		    // send a character over the serialPort is its return false, its not connected
		    while(serialPort.writeString("")) Thread.sleep(2);
		    serialPort.removeEventListener();
		    serialPort.closePort();		    
		}
		catch (SerialPortException ex) {
		    System.out.println("There are an error on writing string to port т: " + ex);
		}
	}
	
	private static class PortReader implements SerialPortEventListener {

		StringBuilder message  = new StringBuilder();
	    @Override
	    public void serialEvent(SerialPortEvent event) {
	            try {
	            	String receivedData = serialPort.readString(event.getEventValue());
	            	message.append(receivedData);
	            	if (receivedData.endsWith("\n")) {	            		
	            		new JSONObject(message.toString());
	            		System.out.print(message.toString());
	            		message.setLength(0);
	            	}
	            }
	            catch (Exception ex) {
	            	message.setLength(0);
//	                System.out.println("Error in receiving string from COM-port: " + ex);
	            }
	       }	
	}
	
	
}

